@@ -1,15 +1,13 @@
 #! -*- encoding: utf-8 -*-
 """
-The core module for the recommendation system. Here is defined the flow for a recommendation request and settings.
+The core module for the recommendation system. Here is defined the flow for a recommendation request and recommendation.settings.
 """
-
-__author__ = "joaonrb"
-
+from __future__ import division, absolute_import, print_function
 import logging
 import numpy as np
 import traceback
 from django.conf import settings
-from recommendation.models import Item, User, TensorCoFi, Popularity, Inventory
+from recommendation.models import Item, User, TensorCoFi, Popularity
 from recommendation.util import initialize
 from recommendation.decorators import ContingencyProtocol
 
@@ -24,8 +22,13 @@ try:
 except KeyError:
     from recommendation.decorators import NoLogger
     logger = NoLogger()
+
+__author__ = "joaonrb"
+
 log_event = logger
 
+MAX_SORT = 1000
+
 
 class IController(object):
     """
@@ -154,7 +157,7 @@ class IController(object):
             apps_idx = [a.pk - 1 for a in user.owned_items.values() if a.pk - 1 <= model.factors[1].shape[0]]
             u_factors = model.online_user_factors(apps_idx)  # New factors for this user
             TensorCoFi.user_matrix[user.pk-1] = u_factors  # store new documentation in Cache
-            return np.squeeze(np.asarray((u_factors * model.factors[1].transpose())))
+            return np.squeeze(np.asarray(np.dot(u_factors, model.factors[1].transpose())))
 
     @log_event(log_event.RECOMMEND)
     def get_recommendation(self, user, n=10):
@@ -180,7 +183,9 @@ class IController(object):
             result = self.get_alternative_recommendation(user)
         for f in self.filters:
             result = f(user, result, size=n)
-        result = [aid+1 for aid, _ in sorted(enumerate(result), key=lambda x: x[1], reverse=True)]
+        max_sort = len(result) if len(result) < MAX_SORT else MAX_SORT
+        top = np.argpartition(-result, max_sort-1)[:max_sort]
+        result = list(top[np.argsort(result[top])[::-1]] + 1)
         for r in self.rerankers:
             result = r(user, result, size=n)
         return result[:n]
@@ -231,7 +236,8 @@ class NotEnoughItemsToCompute(Exception):
     """
     Exception for when user is short in Items to compute a new recommendation
     """
-    pass
+    def __init__(self, *args, **kwargs):
+        super(NotEnoughItemsToCompute, self).__init__(*args, **kwargs)
 
 RECOMMENDATION_ENGINES = {}
 for engine, engine_settings in RECOMMENDATION_SETTINGS.items():
@@ -256,7 +262,7 @@ default_recommendation = RECOMMENDATION_ENGINES["default"]
 
 class ControllerNotDefined(Exception):
     """
-    Exception for when controller is not defined on settings
+    Exception for when controller is not defined on recommendation.settings
     """
     pass
 
