@@ -7,8 +7,8 @@ Diversification models necessary to apply the diversification to a recommendatio
 .. moduleauthor:: joaonrb <joaonrb@gmail.com>
 
 """
-__author__ = "joaonrb"
 
+from __future__ import division, absolute_import, print_function
 import click
 from itertools import chain
 from collections import Counter
@@ -21,6 +21,8 @@ from django.contrib import admin
 from recommendation.models import Item
 from recommendation.decorators import Cached
 
+__author__ = "joaonrb"
+
 
 class Genre(models.Model):
     """
@@ -39,7 +41,7 @@ class Genre(models.Model):
         return self.name
 
     @staticmethod
-    @Cached(cache="local", timeout=60*60)
+    @Cached()
     def get_genre_by_id(genre_id):
         """
         Return the genre belonging to that id with and extra attribute: count_items
@@ -49,7 +51,7 @@ class Genre(models.Model):
         return Genre.objects.filter(pk=genre_id).annotate(count_items=Count("items"))[0]
 
     @staticmethod
-    @Cached(cache="local", timeout=60*60)
+    @Cached()
     def get_all_genres():
         """
         Get all genres
@@ -65,12 +67,13 @@ class Genre(models.Model):
         :return:
         """
         cache = Genre.get_genre_by_id.cache
-        with click.progressbar(Genre.objects.all().annotate(count_items=Count("items")),
-                               label="Loading genres to cache") as bar:
-            for genre in bar:
+        for genre in Genre.objects.all().annotate(count_items=Count("items")):
+        #with click.progressbar(Genre.objects.all().annotate(count_items=Count("items")),
+        #                       label="Loading genres to cache") as bar:
+        #    for genre in bar:
                 Genre.get_genre_by_id.lock_this(
                     cache.set
-                )(Genre.get_genre_by_id.key % genre.pk, genre, Genre.get_genre_by_id.timeout)
+                )(Genre.get_genre_by_id.key(genre.pk), genre, Genre.get_genre_by_id.timeout)
 
         Genre.get_all_genres()
 
@@ -94,7 +97,7 @@ class ItemGenre(models.Model):
         return _("%(item)s's %(genre)s") % {"genre": self.type.name, "item": self.item.external_id}
 
     @staticmethod
-    @Cached(cache="local")
+    @Cached()
     def get_genre_by_item(item_id):
         """
         Get genres list for a specific item
@@ -111,18 +114,20 @@ class ItemGenre(models.Model):
         :return:
         """
         genres = {}
-        with click.progressbar(ItemGenre.objects.all(), label="Loading item by genres to cache") as bar:
-            for item_genre in bar:
+        for item_genre in ItemGenre.objects.all():
+        #with click.progressbar(ItemGenre.objects.all(), label="Loading item by genres to cache") as bar:
+        #    for item_genre in bar:
                 try:
                     genres[item_genre.item_id].append(item_genre.type_id)
                 except KeyError:
                     genres[item_genre.item_id] = [item_genre.type_id]
         cache = ItemGenre.get_genre_by_item.cache
-        with click.progressbar(genres.items(), label="Loading genres by item to cache") as bar:
-            for item_id, genre in bar:
+        for item_id, genre in genres.items():
+        #with click.progressbar(genres.items(), label="Loading genres by item to cache") as bar:
+        #    for item_id, genre in bar:
                 ItemGenre.get_genre_by_item.lock_this(
                     cache.set
-                )(ItemGenre.get_genre_by_item.key % item_id, genre, ItemGenre.get_genre_by_item.timeout)
+                )(ItemGenre.get_genre_by_item.key(item_id), genre, ItemGenre.get_genre_by_item.timeout)
 
     @staticmethod
     def load_item(item):
