@@ -4,6 +4,8 @@ Plugin to remove owned items
 
 """
 
+from __future__ import division, absolute_import, print_function
+
 __author__ = "joaonrb"
 
 
@@ -22,5 +24,8 @@ class FilterOwned(object):
         :return:
         """
         for item in user.owned_items.values():
-            recommendation[item.pk-1] = float("-inf")
+            try:
+                recommendation[item.pk-1] = -1000
+            except IndexError:
+                pass
         return recommendation
\ No newline at end of file
