@@ -27,8 +27,8 @@ Options:
   -h --help                        Show this screen.
   --version                        Show version.
 """
-__author__ = "joaonrb"
 
+from __future__ import division, absolute_import, print_function
 import os
 import sys
 import traceback
@@ -39,14 +39,17 @@ import tarfile
 import json
 import errno
 import shutil
+import click
+import itertools
 from datetime import date, timedelta, datetime
 from django.db.models import Q
 from django_docopt_command import DocOptCommand
 from django.conf import settings
 from recommendation.models import Item, User, Inventory
 from recommendation.diversity.models import Genre, ItemGenre
-from recommendation.language.models import Locale, ItemLocale
-from recommendation.default_settings import TESTING_MODE
+from recommendation.language.models import Locale, ItemLocale, Region, ItemRegion, UserRegion, UserLocale
+
+__author__ = "joaonrb"
 
 
 MOZILLA_DEV_ITEMS_API = "https://marketplace-dev-cdn.allizom.org/dumped-apps/tarballs/%Y-%m-%d.tgz"
@@ -205,30 +208,37 @@ class FillTool(object):
         new_items = {}
         categories = set([])
         locales = set([])
-        for item_eid, json_item in json_items.items():
-            if item_eid not in items:
-                try:
-                    name = json_item["name"][json_item["default_locale"]]
-                except (KeyError, TypeError):
-                    name = json_item.get("name", "NO NAME")
-                except UnicodeEncodeError as e:
-                    logging.error(e)
-                    name = "NO NAME"
-                new_items[item_eid] = Item(external_id=item_eid, name=name)
+        regions = {}
+        with click.progressbar(json_items.items(), label="Loading items") as bar:
+            for item_eid, json_item in bar:
+                if item_eid not in items:
+                    try:
+                        name = json_item["name"][json_item["default_locale"]]
+                    except (KeyError, TypeError):
+                        name = json_item.get("name", "NO NAME")
+                    except UnicodeEncodeError as e:
+                        logging.error(e)
+                        name = "NO NAME"
+                    new_items[item_eid] = Item(external_id=item_eid, name=name)
 
-            # In case of json[self.item_genres_field] = None
-            json_categories = json_item.get(self.item_genres_field, None) or ()
-            if isinstance(json_categories, basestring):
-                categories.add(json_categories)
-            else:
-                categories = categories.union(json_item[self.item_genres_field])
+                # In case of json[self.item_genres_field] = None
+                json_categories = json_item.get(self.item_genres_field, None) or ()
+                if isinstance(json_categories, basestring):
+                    categories.add(json_categories)
+                else:
+                    categories = categories.union(json_item[self.item_genres_field])
 
-            # In case of json[self.item_locales_field] = None
-            json_locales = json_item.get(self.item_locales_field, None) or ()
-            if isinstance(json_locales, basestring):
-                locales.add(json_locales)
-            else:
-                locales = locales.union(json_locales)
+                # In case of json[self.item_locales_field] = None
+                json_locales = json_item.get(self.item_locales_field, None) or ()
+                if isinstance(json_locales, basestring):
+                    locales.add(json_locales)
+                else:
+                    locales = locales.union(json_locales)
+                for region in json_item.get("regions", None) or ():
+                    try:
+                        regions[region["name"]]["items"].append(item_eid)
+                    except KeyError:
+                        regions[region["name"]] = {"slug": region["slug"], "items": [item_eid]}
         #logging.debug("Items ready to be saved")
         Item.objects.bulk_create(new_items.values())
         #logging.debug("%d new items saved with bulk_create" % len(new_items))
@@ -238,16 +248,18 @@ class FillTool(object):
         assert len(items) == len(objects), \
             "Size of items and size of objects are different (%d != %d)" % (len(items), len(objects))
 
-        if "recommendation.diversity" in settings.INSTALLED_APPS and not TESTING_MODE:
+        if "recommendation.diversity" in settings.INSTALLED_APPS:
             #logging.debug("Preparing genres")
             db_categories = self.get_genres(categories)
             self.fill_item_genre(objects, items, db_categories)
             #logging.debug("Genres loaded")
 
-        if "recommendation.language" in settings.INSTALLED_APPS and not TESTING_MODE:
+        if "recommendation.language" in settings.INSTALLED_APPS:
             #logging.debug("Preparing languages")
             db_locales = self.get_locales(locales)
+            db_regions = self.get_regions(regions)
             self.fill_item_locale(objects, items, db_locales)
+            self.fill_item_region(regions, items, db_regions)
             #logging.debug("Locales loaded")
 
     @staticmethod
@@ -269,6 +281,26 @@ class FillTool(object):
                 genres[genre.name] = genre
         return genres
 
+    @staticmethod
+    def get_regions(region_names):
+        """
+        Get regions from database and create the ones that don't exist.
+        :param region_names:
+        :return: A dict with regions mapped to their name
+        """
+        regions = {region.name: region for region in Region.objects.filter(name__in=region_names)}
+        if len(regions) != len(region_names):
+            new_regions = {}
+            for name in region_names:
+                slug = region_names[name]["slug"]
+                if name not in regions:
+                    new_regions[name] = Region(name=name, slug=slug)
+
+            Region.objects.bulk_create(new_regions.values())
+            for region in Region.objects.filter(name__in=new_regions):
+                regions[region.name] = region
+        return regions
+
     @staticmethod
     def get_locales(locales_names):
         """
@@ -348,6 +380,30 @@ class FillTool(object):
                 del item_locales[item_locale.locale_id, item_locale.item_id]
         ItemLocale.objects.bulk_create(item_locales.values())
 
+    @staticmethod
+    def fill_item_region(objects, items, regions):
+        """
+        Fill item locales connection
+        :param items:
+        :param locales:
+        :return:
+        """
+        query_item_regions = Q()
+        item_regions = {}
+        with click.progressbar(objects.items(), label="Loading item regions") as bar:
+            for region_eid, item in bar:
+                region_id = regions[region_eid].pk
+                item_regions[region_id] = {
+                    items[item_eid].pk: ItemRegion(region_id=region_id, item_id=items[item_eid].pk)
+                    for item_eid in item["items"]
+                }
+                query_item_regions = query_item_regions | Q(region_id=region_id, item_id__in=item_regions[region_id])
+
+        if len(query_item_regions) > 0:
+            for item_region in ItemRegion.objects.filter(query_item_regions):
+                del item_regions[item_region.region_id][item_region.item_id]
+        ItemRegion.objects.bulk_create(itertools.chain(*(ir.values() for ir in item_regions.values())))
+
     def clean_tmp(self):
         """
         Remove items from tmp path
@@ -367,7 +423,8 @@ class FillTool(object):
         size = 0
         user_items = {}
         json_users = []
-        import click
+        regions = {}
+        langs = {}
         with click.progressbar(objects, label="Loading users to memory") as bar:
             for obj in bar:
                 if self.user_file_identifier_field in obj:
@@ -381,6 +438,16 @@ class FillTool(object):
                             user_items[item_id] = [user_item]
                     size += 1
                     json_users.append(user_id)
+                    if "region" in obj:
+                        try:
+                            regions[obj["region"]].append(user_id)
+                        except KeyError:
+                            regions[obj["region"]] = [user_id]
+                    if "lang" in obj:
+                        try:
+                            langs[obj["lang"]].append(user_id)
+                        except KeyError:
+                            langs[obj["lang"]] = [user_id]
         logging.debug("Done!")
         users = {user.external_id: user for user in User.objects.filter(external_id__in=json_users)}
         new_users = {}
@@ -396,6 +463,8 @@ class FillTool(object):
         assert len(users) == size, \
             "Size of loaded objects and users in db is not the same (%d != %d)" % (len(users), size)
 
+        if "recommendation.language" in settings.INSTALLED_APPS:
+            self.fill_user_locale(users, regions, langs)
         #logging.debug("%d new users saved with bulk_create" % len(new_users))
 
         #logging.debug("Preparing items")
@@ -447,6 +516,49 @@ class FillTool(object):
                 Inventory.objects.filter(to_delete).delete()
         Inventory.objects.bulk_create(inventory.values())
 
+    @staticmethod
+    def fill_user_locale(users, regions, langs):
+        region_query = Q()
+        user_regions = {}
+        db_regions = {region.slug: region.pk for region in Region.objects.all()}
+        with click.progressbar(regions.items(), label="Load user regions inventory") as bar:
+            for region, ueis in bar:
+                try:
+                    region_id = db_regions[region]
+                except KeyError:
+                    pass
+                else:
+                    region_query = region_query | Q(region_id=region_id, user_id__in=map(lambda x: users[x].pk, ueis))
+                    user_regions[region_id] = {
+                        users[user_eid].pk: UserRegion(user_id=users[user_eid].pk, region_id=region_id)
+                        for user_eid in ueis
+                    }
+        if len(region_query) > 0:
+            for ur in UserRegion.objects.filter(region_query).distinct():
+                del user_regions[ur.region_id][ur.user_id]
+        UserRegion.objects.bulk_create(itertools.chain(*(ur.values() for ur in user_regions.values())))
+
+        locale_query = Q()
+        user_locales = {}
+        db_locales = {str(locale): locale.pk for locale in Locale.objects.all()}
+        with click.progressbar(langs.items(), label="Load user locales inventory") as bar:
+            for locale, ueis in bar:
+                try:
+                    locale_id = db_locales[locale]
+                except KeyError:
+                    pass
+                else:
+                    locale_query = locale_query | Q(locale_id=locale_id, user_id__in=map(lambda x: users[x].pk, ueis))
+                    user_locales[locale_id] = {
+                        users[user_eid].pk: UserLocale(user_id=users[user_eid].pk, locale_id=locale_id)
+                        for user_eid in ueis
+                    }
+
+        if len(locale_query) > 0:
+            for ur in UserLocale.objects.filter(locale_query).distinct():
+                del user_locales[ur.locale_id][ur.user_id]
+        UserLocale.objects.bulk_create(itertools.chain(*(ur.values() for ur in user_locales.values())))
+
 
 class Command(DocOptCommand):
     docs = __doc__
