@@ -3,16 +3,12 @@
 Models for the base of the recommendation system. The base of the recommendation system makes uses of the user, item
 amd connection between them.
 """
-__author__ = "joaonrb"
-
-
+from __future__ import division, absolute_import, print_function
 import sys
 import base64
 import numpy as np
 import pandas as pd
-import click
-import pandas.io.sql as psql
-from django.db import connection
+# import click
 from django.db import models
 from django.utils.translation import ugettext as _
 from django.utils.six import with_metaclass
@@ -25,6 +21,8 @@ from recommendation.decorators import Cached
 if sys.version_info >= (3, 0):
     basestring = unicode = str
 
+__author__ = "joaonrb"
+
 
 class NPArrayField(with_metaclass(models.SubfieldBase, models.TextField)):
     """
@@ -101,8 +99,9 @@ class NPArrayField(with_metaclass(models.SubfieldBase, models.TextField)):
         :param value: Matrix to keep in database
         :return: Base64 representation string encoded in utf-8
         """
-        return ":".join([str(len(value.shape)), ":".join(map(lambda x: str(x), value.shape)),
-                         base64.b64encode(value.tostring())])
+        if isinstance(value, np.ndarray):
+            return ":".join([str(len(value.shape)), ":".join(map(lambda x: str(x), value.shape)),
+                            base64.b64encode(value.tostring())])
 
 
 class Item(models.Model):
@@ -141,10 +140,10 @@ class Item(models.Model):
         """
         Item.get_item_by_external_id.lock_this(
             Item.get_item_by_external_id.cache.set
-        )(Item.get_item_by_external_id.key % self.external_id, self, Item.get_item_by_external_id.timeout)
+        )(Item.get_item_by_external_id.key(self.external_id), self, Item.get_item_by_external_id.timeout)
         Item.get_item_external_id_by_id.lock_this(
             Item.get_item_external_id_by_id.cache.set
-        )(Item.get_item_external_id_by_id.key % self.pk, self.external_id, Item.get_item_external_id_by_id.timeout)
+        )(Item.get_item_external_id_by_id.key(self.pk), self.external_id, Item.get_item_external_id_by_id.timeout)
 
     def del_item_from_cache(self):
         """
@@ -152,10 +151,10 @@ class Item(models.Model):
         """
         Item.get_item_by_external_id.lock_this(
             Item.get_item_by_external_id.cache.delete
-        )(Item.get_item_by_external_id.key % self.external_id)
+        )(Item.get_item_by_external_id.key(self.external_id))
         Item.get_item_external_id_by_id.lock_this(
             Item.get_item_external_id_by_id.cache.delete
-        )(Item.get_item_external_id_by_id.key % self.pk)
+        )(Item.get_item_external_id_by_id.key(self.pk))
 
     class Meta:
         verbose_name = _("item")
@@ -169,8 +168,9 @@ class Item(models.Model):
 
     @staticmethod
     def load_to_cache():
-        with click.progressbar(Item.objects.all(), label="Loading items to cache") as bar:
-            for item in bar:
+        for item in Item.objects.all():
+        #with click.progressbar(Item.objects.all(), label="Loading items to cache") as bar:
+        #    for item in bar:
                 item.put_item_to_cache()
 
 
@@ -237,7 +237,7 @@ class User(models.Model):
         return User.get_user_by_id(User.get_user_id_by_external_id(external_id))
 
     @staticmethod
-    @Cached()
+    @Cached(cache="owned_items", lock_id=0)
     def get_user_items(user_id):
         """
         Get user items
@@ -283,32 +283,36 @@ class User(models.Model):
 
     @staticmethod
     def load_to_cache():
-        with click.progressbar(User.objects.all(), label="Loading users to cache") as bar:
-            for user in bar:
+        for user in User.objects.all():
+        #with click.progressbar(User.objects.all(), label="Loading users to cache") as bar:
+        #    for user in bar:
                 User.get_user_by_id.lock_this(
                     User.get_user_by_id.cache.set
-                )(User.get_user_by_id.key % user.pk, user, User.get_user_by_id.timeout)
-            User.get_user_id_by_external_id.lock_this(
-                User.get_user_id_by_external_id.cache.set
-            )(User.get_user_id_by_external_id.key % user.external_id, user.pk, User.get_user_id_by_external_id.timeout)
+                )(User.get_user_by_id.key(user.pk), user, User.get_user_by_id.timeout)
+                User.get_user_id_by_external_id.lock_this(
+                    User.get_user_id_by_external_id.cache.set
+                )(User.get_user_id_by_external_id.key(user.external_id), user.pk,
+                  User.get_user_id_by_external_id.timeout)
+
+    @staticmethod
+    def load_owned_items():
         lenght = Inventory.objects.all().count()
-        with click.progressbar(range(0, lenght, 100000),
-                               label="Loading owned items to cache") as bar:
-            inventory = {}
-            max_id = 0
-            for i in bar:
-                for max_id, user_id, item_id, is_dropped in Inventory.objects.filter(id__gt=max_id)\
-                        .order_by("pk")[i:i+100000].values_list("pk", "user_id", "item_id", "is_dropped"):
-                    try:
-                        inventory[user_id][item_id] = is_dropped
-                    except KeyError:
-                        inventory[user_id] = {
-                            item_id: is_dropped
-                        }
+        inventory = {}
+        max_id = 0
+        for i in xrange(0, lenght, 100000):
+            #with click.progressbar(range(0, lenght, 100000),
+            #                       label="Loading owned items to cache") as bar:
+            #    for i in bar:
+            for max_id, user_id, item_id, is_dropped in Inventory.objects.filter(id__gt=max_id) \
+                    .order_by("pk")[i:i+100000].values_list("pk", "user_id", "item_id", "is_dropped"):
+                try:
+                    inventory[user_id][item_id] = is_dropped
+                except KeyError:
+                    inventory[user_id] = {item_id: is_dropped}
             for ueid, items in inventory.items():
                 User.get_user_items.lock_this(
                     User.get_user_items.cache.set
-                )(User.get_user_items.key % ueid, items, User.get_user_items.timeout)
+                )(User.get_user_items.key(ueid), items, User.get_user_items.timeout)
 
     def load_user(self):
         """
@@ -316,10 +320,10 @@ class User(models.Model):
         """
         User.get_user_by_id.lock_this(
             User.get_user_by_id.cache.set
-        )(User.get_user_by_id.key % self.pk, self, User.get_user_by_id.timeout)
+        )(User.get_user_by_id.key(self.pk), self, User.get_user_by_id.timeout)
         User.get_user_id_by_external_id.lock_this(
             User.get_user_id_by_external_id.cache.set
-        )(User.get_user_id_by_external_id.key % self.external_id, self.pk, User.get_user_id_by_external_id.timeout)
+        )(User.get_user_id_by_external_id.key(self.external_id), self.pk, User.get_user_id_by_external_id.timeout)
         User.get_user_items(self.pk)
 
     def delete_user(self):
@@ -328,34 +332,34 @@ class User(models.Model):
         """
         User.get_user_by_id.lock_this(
             User.get_user_by_id.cache.delete
-        )(User.get_user_by_id.key % self.external_id)
+        )(User.get_user_by_id.key(self.external_id))
         User.get_user_id_by_external_id.lock_this(
             User.get_user_id_by_external_id.cache.delete
-        )(User.get_user_id_by_external_id.key % self.external_id)
+        )(User.get_user_id_by_external_id.key(self.external_id))
         User.get_user_items.lock_this(
             User.get_user_items.cache.delete
-        )(User.get_user_items.key % self.external_id)
+        )(User.get_user_items.key(self.external_id))
 
     def load_item(self, entry):
         """
         Load a single inventory entry
         """
         cache = User.get_user_items.cache
-        entries = cache.get(User.get_user_items.key % self.pk, {})
+        entries = cache.get(User.get_user_items.key(self.pk), {})
         entries[entry.item_id] = entry.is_dropped
-        cache.set(User.get_user_items.key % self.pk, entries)
+        cache.set(User.get_user_items.key(self.pk), entries)
 
     def delete_item(self, entry):
         """
         Load a single inventory entry
         """
         cache = User.get_user_items.cache
-        entries = cache.get(User.get_user_items.key % self.pk, {})
+        entries = cache.get(User.get_user_items.key(self.pk), {})
         try:
             del entries[entry.item.pk]
         except KeyError:
             pass
-        cache.set(User.get_user_items.key % self.pk, entries)
+        cache.set(User.get_user_items.key(self.pk), entries)
 
 
 @receiver(post_save, sender=User)
@@ -472,7 +476,7 @@ class UserMatrix:
                 dec = UserMatrix.get_user_array
                 dec.lock_this(
                     dec.cache.set
-                )(dec.key % index, value, dec.timeout)
+                )(dec.key(index), value, dec.timeout)
         except User.DoesNotExist:
             pass
 
@@ -480,7 +484,7 @@ class UserMatrix:
         dec = UserMatrix.get_user_array
         dec.lock_this(
             dec.cache.delete
-        )(dec.key % index)
+        )(dec.key(index))
 
 
 class FactorsContainer:
@@ -544,15 +548,16 @@ class TensorCoFi(PyTensorCoFi):
         except IndexError:
             raise NotCached("TensorCoFi not in db")
 
-        with click.progressbar(enumerate(users.numpy),
-                               length=users.numpy.shape[0],
-                               label="Loading TensorCoFi users to cache") as bar:
-            for i, u in bar:
+        for i, u in enumerate(users.numpy):
+        #with click.progressbar(enumerate(users.numpy),
+        #                       length=users.numpy.shape[0],
+        #                       label="Loading TensorCoFi users to cache") as bar:
+        #    for i, u in bar:
                 TensorCoFi.user_matrix[i] = u
         TensorCoFi.get_item_matrix()
 
     @staticmethod
-    @Cached(cache="local")
+    @Cached()
     def get_item_matrix():
         try:
             items = Matrix.objects.filter(name="tensorcofi", model_id=1).order_by("-id")[0]
@@ -561,14 +566,13 @@ class TensorCoFi(PyTensorCoFi):
         return items.numpy
 
     @staticmethod
-    @Cached(cache="local")
+    @Cached()
     def get_model_from_cache(*args, **kwargs):
         tensor = TensorCoFi(n_users=User.objects.aggregate(max=models.Max("pk"))["max"],
                             n_items=Item.objects.aggregate(max=models.Max("pk"))["max"])
         tensor.factors = FactorsContainer(tensor)
         return tensor
 
-
     @staticmethod
     def get_model(*args, **kwargs):
         return TensorCoFi.get_model_from_cache(args, **kwargs).factors
@@ -639,7 +643,7 @@ class Popularity(TestFMPopularity):
             self.get_user_column(): MySQLMapDummy(),
             self.get_item_column(): MySQLMapDummy()
         }
-        self.popularity_recommendation = []
+        self.popularity_recommendation = None
 
     def fit(self, training_data):
         """
@@ -648,11 +652,7 @@ class Popularity(TestFMPopularity):
         :return:
         """
         super(Popularity, self).fit(training_data)
-        self.popularity_recommendation = []
-        for i in range(self.n_items):
-            self.popularity_recommendation.append(self._counts.get(i+1, 0.0))
-
-        self.popularity_recommendation = np.array(self.popularity_recommendation)
+        self.popularity_recommendation = np.array([self._counts.get(i+1, 0.0) for i in xrange(self.n_items)])
 
     @property
     def recommendation(self):
@@ -661,19 +661,24 @@ class Popularity(TestFMPopularity):
     @recommendation.setter
     def recommendation(self, value):
         self.popularity_recommendation = value.tolist()
-        self._counts = {i+1: value[i] for i in range(self.n_items)}
+        self._counts = {i+1: value[i] for i in xrange(self.n_items)}
 
     @staticmethod
-    @Cached(cache="local")
+    @Cached()
     def load_popularity():
-        model = Popularity(n_items=Item.objects.all().count())
+        model = Popularity(n_items=Item.objects.aggregate(max=models.Max("pk"))["max"])
         pop = Matrix.objects.filter(name="popularity").order_by("-id")[0]
         model.recommendation = pop.numpy
         return model
 
     @staticmethod
     def load_to_cache():
-        return Popularity.load_popularity()
+        model = Popularity(n_items=Item.objects.aggregate(max=models.Max("pk"))["max"])
+        pop = Matrix.objects.filter(name="popularity").order_by("-id")[0]
+        model.recommendation = pop.numpy
+        Popularity.load_popularity.lock_this(
+            Popularity.load_popularity.cache.set
+        )(Popularity.load_popularity.key(), model, Popularity.load_popularity.timeout)
 
     @staticmethod
     def get_model():
@@ -691,7 +696,7 @@ class Popularity(TestFMPopularity):
         Train the popular model
         :return:
         """
-        popular_model = Popularity(n_items=Item.objects.all().count())
+        popular_model = Popularity(n_items=Item.objects.aggregate(max=models.Max("pk"))["max"])
         users, items = zip(*Inventory.objects.all().values_list("user_id", "item_id"))
         data = pd.DataFrame({"item": items, "user": users})
         popular_model.fit(data)
