@@ -4,14 +4,12 @@ A decorator to register events into log. Created on Fev 11, 2014
 
 """
 
-__author__ = "joaonrb"
-
-from recommendation.simple_logging.models import LogEntry, LOGGER_MAX_LOGS
+from __future__ import division, absolute_import, print_function
+from recommendation.simple_logging.models import LogEntry
 from recommendation.decorators import ILogger
-from recommendation.models import Item, User
-from recommendation.decorators import GoToThreadQueue
 import functools
-from collections import deque
+
+__author__ = "joaonrb"
 
 
 class LogEvent(ILogger):
@@ -32,7 +30,6 @@ class LogEvent(ILogger):
         else:
             self.do_call = self.std
 
-    @GoToThreadQueue()
     def bulk_load(self, user, recommendation):
         new_logs = [
             LogEntry(user=user, item_id=iid, type=self.log_type, value=i)
